import MT as mt
import pandas as pd
from time import sleep
import os
from dotenv import load_dotenv
import time
import socket
import ta

load_dotenv()

folder_dir = os.getenv('DIR')
login = int(os.getenv('LOGIN'))
pwd = os.getenv('PWD')
server = os.getenv('SERVER')
symbol = os.getenv('SYMBOL')
path = "C:/Program Files/" + folder_dir +"/terminal64.exe"
strategy_name = "Candlestick Bot"

# Create by: Admin | Forex In Thai 
# https://www.forexinthai.com
# https://www.forexinthai.com/105121/




def market_order(symbol, volume, order_type, deviation, magic, stoploss, take_profit,strategy_name):

    order_type_dict = {
        'buy': mt.ORDER_TYPE_BUY,
        'sell': mt.ORDER_TYPE_SELL
    }

    price_dict = {
        'buy': mt.symbol_info_tick(symbol).ask,
        'sell': mt.symbol_info_tick(symbol).bid
    }

    request = {
        "action": mt.TRADE_ACTION_DEAL,
        "symbol": symbol,
        "volume": volume,  # FLOAT
        "type": order_type_dict[order_type],
        "price": price_dict[order_type],
        "sl": stoploss,  # FLOAT
        "tp": take_profit,  # FLOAT
        "deviation": deviation,  # INTERGER
        "magic": magic,  # INTERGER
        "comment": strategy_name,
        "type_time": mt.ORDER_TIME_GTC,
        "type_filling": mt.ORDER_FILLING_IOC,  # mt.ORDER_FILLING_FOK if IOC does not work
    }

    order_result = mt.order_send(request)
    return (order_result)
    
#sell
def is_bearish_evening_star(df):
    # เงื่อนไขสำหรับแท่งเทียนแต่ละวัน
    first_candle_bullish = df['close2'] > df['open2']
    small_body_second_day = abs(df['close1'] - df['open1']) <= (df['high1'] - df['low1']) / 3
    third_candle_bearish = df['close'] < df['open']
    
    # ตำแหน่งของแท่งเทียนวันที่สองและสาม
    second_candle_gap_up = df['open1'] > df['close2']
    third_candle_closes_below_first_midpoint = df['close'] < (df['open2'] + (df['close2'] - df['open2']) / 2)
    
    # ตรวจสอบทั้งสามแท่งเทียน
    evening_star = first_candle_bullish & small_body_second_day & second_candle_gap_up & third_candle_bearish & third_candle_closes_below_first_midpoint
    return evening_star
    
#buy
def is_morning_star(df):

    # เงื่อนไขสำหรับแท่งเทียนแต่ละวัน
    first_candle_bearish = df['close2'] > df['open2']
    small_body_second_day = abs(df['close1'] - df['open1']) <= (df['high1'] - df['low1']) / 3
    third_candle_bullish = df['close'] > df['open']
    
    # ตำแหน่งของแท่งเทียนวันที่สองและสาม
    second_candle_gap_down = df['open1'] < df['close2']
    third_candle_closes_above_first_midpoint = df['close'] > (df['open2'] + (df['close2'] - df['open2']) / 2)
    
    # ตรวจสอบทั้งสามแท่งเทียน
    morning_star = first_candle_bearish & small_body_second_day & second_candle_gap_down & third_candle_bullish & third_candle_closes_above_first_midpoint
    return morning_star
    



def get_signal(symbol, timeframe):
    global strategy_name
    
    rates = mt.copy_rates_from_pos(symbol, timeframe, 1, 100)

    # Create a DataFrame
    df = pd.DataFrame(rates)
    df['time'] = pd.to_datetime(df['time'], unit='s')
    df.set_index('time', inplace=True)
    df['rsi'] = ta.momentum.rsi(df['close'], window=14)
    rsi = df.iloc[-1];
 
    ohlc = mt.copy_rates_from_pos(symbol, timeframe, 1, 3)
    ohlc_df = pd.DataFrame(ohlc)[['time', 'open', 'high', 'low', 'close']]
    ohlc_df['time'] = pd.to_datetime(ohlc_df['time'], unit='s')

    ohlc_df['open2'] = ohlc_df['open'].shift(2)     
    ohlc_df['high2'] = ohlc_df['high'].shift(2)     
    ohlc_df['low2'] = ohlc_df['low'].shift(2)       
    ohlc_df['close2'] = ohlc_df['close'].shift(2)   
    
    
    ohlc_df['open1'] = ohlc_df['open'].shift(1)     
    ohlc_df['high1'] = ohlc_df['high'].shift(1)     
    ohlc_df['low1'] = ohlc_df['low'].shift(1)       
    ohlc_df['close1'] = ohlc_df['close'].shift(1)   
    ohlc_df['rsi'] = rsi['rsi']
    
   
    candle = ohlc_df.iloc[-1];
    
    if candle['high1'] - candle['low1'] == 0:
        return False
    
   

    is_buy = False
    is_sell = False
    

    print(f"----------------------------------------")
    print(f" Check All ")
    print(f"----------------------------------------")
    
    buy_patterns = {
    "morning_star": is_morning_star(candle)
    }

    
    for key, pattern in buy_patterns.items():
        print(f"{key}: {pattern}")
        if pattern == True:
            is_buy = True
            strategy_name = key
            break
            
    sell_patterns = {
    "bearish_evening_star": is_bearish_evening_star(candle)
    }

    
    for key, pattern in sell_patterns.items():
        print(f"{key}: {pattern}")
        if pattern == True:
            is_sell = True
            strategy_name = key
            break        

    print(f"----------------------------------------")
    print(f" BUY: {is_buy}")
    print(f"----------------------------------------")

    print(f"----------------------------------------")
    print(f" SELL: {is_sell}")
    print(f"----------------------------------------")
    

    #print(candle)
    print(ohlc_df)
    
    if is_buy:
        return 'buy'
    elif is_sell:
        return 'sell'
    else:
        return False
        


if __name__ == '__main__':
    # connect to MetaTrader5 Terminal
    if not mt.initialize(path=path,login=login, server=server,password=pwd):
        print("Initialize() failed, error code =", mt.last_error())
        quit()

    print(" _____                   ___       _____ _           _")
    print("|  ______  _ __ _____  _|_ _|_ __ |_   _| |__   __ _(_)")
    print("| |_ / _ \| '__/ _ \ \/  | || '_ \  | | | '_ \ / _` | |")
    print("|  _| (_) | | |  __/>  < | || | |   | | | | | | (_| | |")
    print("|_|  \___/|_|  \___/_/\_|___|_| |_  |_| |_| |_|\__,_|_|")
    print("")
    
    timeframe = mt.TIMEFRAME_M1
    volume = 0.02
    deviation=20
    magic=30
    stoploss=0.0
    take_profit=0.0
    sl = 400
    tp = 400
    

    trading_allowed = True
    while trading_allowed:
        # Careful! Loop can open infinite positions!

        signal = get_signal(symbol, timeframe)
        print('signal', signal)
        print('########################################################################################################\n')

        if signal == 'buy' and mt.positions_total() == 0:
            price = mt.symbol_info_tick(symbol).ask
            point = mt.symbol_info(symbol).point
            stoploss = price - sl * point
            take_profit = price + tp * point
            res = market_order(symbol, volume, 'buy',deviation, magic, stoploss, take_profit,strategy_name)
            trading_allowed = False

        elif signal == 'sell' and mt.positions_total() == 0:
            point = mt.symbol_info(symbol).point
            price = mt.symbol_info_tick(symbol).bid
            stoploss = price + sl * point
            take_profit = price - tp * point
            res = market_order(symbol, volume, 'sell',deviation, magic, stoploss, take_profit,strategy_name)
            trading_allowed = False

        sleep(1)


